//modified by Yusuke Sato 2017/11/22
#include <iostream>
#include <fstream>
#include <cmath>
#include <cstdlib>

using namespace std;

// ========= DEFINE CONSTANTS =========
// beam energy in MeV
double energy = 26.0;

// define world region
const double x_min = -3000.;
const double x_max =  3000.;
const double y_min = -3000.;
const double y_max =  3000.;

// define plot region
const double x_min_plot = -100.;
const double x_max_plot =  100.;
const double y_min_plot = -100.;
const double y_max_plot =  100.;

// define constants
const double m_alpha = 3727.379; // alpha mass in MeV

const double alpha = 1.0/137.036; // fine-structure constant
const double hc = 197.327;        // in MeV.fm

const double z_alpha = 2.0;
const double z_au    = 79.0;

//2ABgr no yatu
const double V0 = 1000 ;//MeV?wakaran 
const double R = 6.69;//fm,text no yatu r*a**(1/3)niyoru
const double a = 1/sqrt(R);

// define step in Runge-Kutta (in fm/c unit)
const double step=10.0;

// functions
void solve(double, double*, double*);
void solve_1step(double, double*, double*);

double potential(double);
double getforce(double);

void BtoTheta(double, double*, double*);
void SolveCrossSection(double *, double *, double *, double *);

// ========= MAIN FUNCTION =========
int main(void)
{

  ofstream fout_datas;
  fout_datas.open("./gensan_ScatAngleGen.dat");
  if(!fout_datas){
    cout << "cannot open data files." << endl;
    return 1;
  }

  //for b - theta conncetionship
  double b_value[100];
  double b_step = 0.1 ;
  double theta_value[100];
  double ThetaOverB[100];
  double crosssection[100];

  //set up b
  int i_bSetUp = 0;
  while(i_bSetUp<100){
    b_value[i_bSetUp] = 10.0 - b_step*(double)i_bSetUp;
    i_bSetUp++;
  }


  //for each b theta
  int i_solvetheta = 0;
  
  while(i_solvetheta<100){

  // check input
  // irane

    cout << endl;
    cout << "  Alpha beam energy = " << energy << " MeV" << endl;
    cout << "  Impact paramter   = " << b_value[i_solvetheta] << " fm" << endl;
  
  
  // open output file 
  //ofstream fout;
  //fout.open("./output.gp");
  //if(!fout){
    //cout << "cannot open file!!" << endl;
    //return 1;
  //}

  //open test file 


  

  // for Gnuplot
  //fout << "set xrange [" << x_min_plot << ":" << x_max_plot << "]" << endl;
  //fout << "set yrange [" << y_min_plot << ":" << y_max_plot << "]" << endl;
  //fout << "plot \"-\" using 1:2 with line notitle" << endl;

  
  // for Runge-Kutta
  double x[4];      // x, y, vx, vy
  double x_next[4]; // x, y, vx, vy

  // set initial position and velocity
  x[0] = x_min;          // x0
  x[1] = b_value[i_solvetheta];  // y0 (impact paramter)
  x[2] = sqrt(2.0*energy/m_alpha); // v_x0
  x[3] = 0;                        // v_y0

  // do loop using Runge-Kutta method
  while(x[0]>=x_min && x[0]<=x_max && x[1]>=y_min && x[1]<y_max){
    //fout << x[0] << " " << x[1] << endl;
    solve(step, x, x_next);
    for(int i=0;i<4;i++){
      x[i] = x_next[i];
    }
  }

  // print scattering angle
  // write parameters into output file
  cout << endl;
  if(x[2]>0){
    cout << "  Scatteing Angle   = " << atan(x[3]/x[2])/3.14*180 << " deg" << endl;
    fout_datas << b_value[i_solvetheta] << " " << atan(x[3]/x[2])/3.14*180 << endl;
    theta_value[i_solvetheta] = atan(x[3]/x[2])/3.14*180;
  }else{
    cout << "  Scatteing Angle   = " << atan(x[3]/x[2])/3.14*180+180 << " deg" << endl;
    fout_datas << b_value[i_solvetheta] << " " << atan(x[3]/x[2])/3.14*180+180 << endl;
    theta_value[i_solvetheta] = atan(x[3]/x[2])/3.14*180+180;
  }
  cout << endl;

  // close file
  //fout << "EOF" << endl;
  //fout_datas << "EOF" << endl;
  //fout.close();
  i_solvetheta++;
  }
  BtoTheta(b_step,theta_value ,ThetaOverB);
  SolveCrossSection(b_value,theta_value,ThetaOverB,crosssection);
  fout_datas.close();
  return 0;
}

void solve(double step, double *x, double *x_next)
{
  double x_local[4];
  double k1_x[4], k2_x[4], k3_x[4], k4_x[4];
  
  for(int i=0;i<4;i++) x_local[i] = x[i];
  solve_1step(step,x_local,k1_x);
  
  for(int i=0;i<4;i++) x_local[i] = x[i] + 0.5*k1_x[i];
  solve_1step(step,x_local,k2_x);

  for(int i=0;i<4;i++) x_local[i] = x[i] + 0.5*k2_x[i];
  solve_1step(step,x_local,k3_x);

  for(int i=0;i<4;i++) x_local[i] = x[i] + k3_x[i];
  solve_1step(step,x_local,k4_x);

  for(int i=0;i<4;i++)
    x_next[i] = x[i] + (k1_x[i] + 2.0*k2_x[i] + 2.0*k3_x[i] + k4_x[i])/6.0;
  
}

void solve_1step(double step, double *x, double* k_x)
{
  
  k_x[0] = step * x[2];
  k_x[1] = step * x[3];

  double rr = sqrt(x[0]*x[0] + x[1]*x[1]);
  k_x[2] = step * x[0]/rr * getforce(rr) / (m_alpha);
  k_x[3] = step * x[1]/rr * getforce(rr) / (m_alpha);
  
}

// define potential here
double potential(double x)
{
  // Coulomb potential between alpha and Au
  //double factor = alpha * hc * z_alpha * z_au;
  return V0/(1 + exp((x - R)/a));
}

// get central difference quatient to estimate of derivative of the potential
double getcdq(double x, double dx)
{
  return (potential(x+dx) - potential(x-dx)) / (2*dx);
}

// get force from derivative of potential using Richardson's extrapolation method
double getforce(double x)
{

  double dx = 0.001;

  double f = - (4*getcdq(x,dx/2) - getcdq(x,dx)) / 3;
  
  return f;
  
}

void BtoTheta(double b_kizami, double *theta , double *thetaoverb)
{
  ofstream fout;
  fout.open("./gensan_Bikeisuu.dat");
  if(!fout){
    cout << "cannot open Bikeisuu files. Some process might be using it." << endl;
  }
  //nakami ha itizi kinzi no suuti bibun
  int i=1;
  while(i<100){
    thetaoverb[i] = (theta[i + 1] - theta[i - 1])/(2 * b_kizami);
    fout << theta [i] << " " << thetaoverb[i] << endl;
    i++;
  }
}

void SolveCrossSection(double *b, double *theta, double *dbdth, double *cs){
  int i = 0;
  while (i < 100){
    cs[i] = b[i]/(abs(sin(theta[i]*3.14/180))*dbdth[i]);
    i++;
  }
  ofstream fout;
  fout.open("./gensan_Crosssection.dat");
  int j_write = 0;
  while(j_write < 100){
    fout << theta[j_write] << " " << cs[j_write] << endl;
    j_write++;
  }
}
